# Case Profile: ORG A

## Organization Characteristics

- **Domain:** Involved in the "data business", and frequently collects and processes large amounts of crowdsourced data for mobile applications.
- **Organization Size:** 50-100 employees / Small according to [StatCanada](https://web.archive.org/web/20200604201858/https://www150.statcan.gc.ca/n1/pub/11f0027m/2011069/part-partie1-eng.htm)
- **Team(s):** 4 (whose members we've interviewed) 
- **Number of Interviewees:** 5
- **Average Team Size:** 3-4
- **Distribution:** Employees are co-located.
- **Continuous Status:** ORG A classifies itself as an organization that follows continuous practices. They have incorporated continuous tools and practices into their developer workflow.
- **Continuous Integration Infrastructure:** ORG A uses Jenkins as its primary tool for build automation, with Docker to manage container configuration, and they have recently started migrating to Kubernetes (a container-based infrastructure management system) to manage their infrastructure.

## Background

In this section, we attempt to provide context around the current workflow, including why ORG A decided to move to the continuous methodology, what we understand to be ORG A's interpretation of the continuous methodology through our interviews, and what makes ORG A unique with respect to other organizations within our study. 

### Reasons for adopting the continuous methodology

ORG A adopted Jenkins early on (around 8 years ago) to ensure consistency and reproducibility.
This was necessary because of the nature of the service ORG A provides in collecting, analyzing, and making vast amounts of real-time data available.
Any changes to their data-processing pipeline needed to be verified before being incorporated into its live version.
And to mitigate the effects local development environments would have on the changes made by developers, a neutral build environment was the next step.
Tests would no longer run in differently configured environments, because the environments themselves would be kept in sync by Docker.
And changes would also be tested in a consistent environment using Jenkins.

### Interpretation of the continuous methodology

When we first asked participants from ORG A about their implementation of continuous practices, four out of the five participants responded with tool-specific answers in the sense that continuous practices essentially revolve around automated builds.
They then began describing how the tools they use (in this case, Jenkins and Docker) facilitated building projects and reduced overhead.
The remaining participant was not familiar with the continuous methodology, and instead described project code design and architecture.

### Unique aspect: Separation between dev and ops

Generally speaking, there is a separation between the dev and ops roles in ORG A.
For most projects, developers are not involved with operating the infrastructure required for their applications to run, nor are they involved with the "live" running of their services.
The exception to this is projects 1 and 2, where they are currently moving to a developer-managed Kubernetes cluster (project 1) and an internal report generation service that does not require a dedicated infrastructure to run "live" (project 2).
This separation between development and operations has caused some friction, particularly within the team working on project 1 and the ops team.

### Teams we've interviewed

- **Team 1:** This team focuses on user-facing applications. At the time of the interviews, they were working on website development and migrating their infrastructure to a developer-managed Kubernetes system.
- **Team 2:** This team is responsible for the development and maintenance of ORG A's data processing pipeline, from data ingress on participating devices to data egress on the customer-facing side of things.
- **Team 3:** This team focuses on web-based development for the purpose of monthly customer-facing reporting based on the processed data.
- **Team 4:** This team works on another infrastructure-related project whose details were not shared with us.

## Current Status

Throughout this section, we go over the reported benefits, possible problems and trade-offs, and recommendations regarding ORG A's current workflow. Afterwards, we go into more detail regarding reported developer workflow as per the interviews we conducted. Finally, we compare that workflow based on the continuous practices we are investigating as originally drafted by [Fowler](https://web.archive.org/web/20200522100521/https://martinfowler.com/articles/originalContinuousIntegration.html).

### Benefits attributed to the continuous practices in place

- Ease of change coordination across project components due to repository structure.
- Increased velocity, consistency, and repeatability due to automation.
- Reduced context-switching between project tasks due to short builds.
- Making it easy for anyone to get the latest executable using Docker has greatly facilitated development environment setup, minimizing the amount of time and effort developers need to spend configuring their local systems.
- Using Slack to communicate build status (as well as BitBucket, emails, and Jenkins) facilitates keeping everyone in the loop.

### Possible problems related to the practices in place

- Overhead created by having to maintain build artifacts.
- Possible vendor lock-in by using unique Jenkins configurations.
- Development cadence hampered by reviewers becoming bottlenecks.
- Increased build times due to the inclusion of data processing tasks in the build.
- Added overhead of having to build any component images used in your container when using Docker.
- Using Slack to communicate build status (as well as BitBucket, emails, and Jenkins) may result in channel overload whereby people stop paying attention to notifications.

### Trade-offs encountered when applying a practice

- It is easier to manually test changes than it is to include automated tests for each change; Less effort is required for manual testing.

### Recommendations

- Cadence might be affected by the reviewer model where developers require a reviewer's approval before merging and deploying changes. Another organization we interviewed is currently doing away with this, and their developer-ownership situation prevents context loss when being asked to deploy/maintain their changes.
- Jenkins configuration files could be restructured to rely on scripts as opposed to having unique properties/commands. This might also facilitate migration to different tools.
- Build time could be improved by minimizing the build process's reliance on external services, as those take up time, and introduce external points of failure.
- Building discipline when monitoring build system notifications could be helped by reducing the number of channels these notifications come across. For instance, if you're using GitHub both as a repository as well as a build system, notifications are centralized in one location as opposed to having them distributed across several tools.

### Detailed workflow

We grouped the information below from all four teams whose members we interviewed, allowing us to construct a model of how the organization as a whole applies the continuous practices we are investigating. We find that ORG A follows continuous practices to varying degrees. Even more interesting, the application of continuous practices varies across all four teams.

#### Typical developer workflow

Developers usually work on separate projects.
Project repositories contain the source code for a particular project, in addition to the in-house dependencies needed for it to build and run successfully, as well as a build configuration file (either Jenkins or Docker files).
Due to the way project repositories and programming tasks are structured, there are few integration issues because developers generally work on either separate modules within the same project or on separate projects altogether.
Once a developer commits their changes, they open a pull request.
This triggers a build that runs tests on that pull request and reports the result in Jenkins, BitBucket, and Slack.
Afterwards, a dedicated reviewer either approves or rejects the pull request, and the changes are either merged in or updated to conform with the review.
The changes are then deployed to a _scratch cluster_, or staging area, for a team lead to review.
If a developer wishes to deploy their changes to production, a team lead must trigger the deployment process which is typically done on a weekly or bi-weekly basis, depending on the project.

#### Comparison against Fowler's continuous practices

**Maintaining a single source repository:** For _team 1_, the project is logically separated into several repositories. Because this is a large and complex application, this choice of separation was made to facilitate dealing with changes in the form of pull requests. This way, like changes (e.g., those made to the backend) can be grouped together into a single pull request. For _teams 2 and 4_, several repositories feed into a single repository to which the pipeline is attached in a star-like model. Dependencies developed within the organization are logically separated into their own projects and kept within separate repos. They are linked to the master repository via dependency management tools and are used in the build process later in the pipeline. This organization of repositories is due to most of the dependencies starting out as proofs of concept, which are incorporated into the master project once they've been proven to work. For _team 3_, the project and all its dependencies are grouped together into the same repository because all components of that project are tightly linked to each other and do not have any uses outside of this particular project.

**Automating the build:** All four projects involve automated builds. The builds are run by Jenkins with the help of Docker for the reasons stated earlier: to guarantee consistency and reproducibility. Furthermore, one participant mentioned that this automation increases development velocity and cuts down on "busy work" which frees up developers to focus on more business-critical and creative tasks, as opposed to manually performing repeatable tasks. A project repository typically contains a Jenkins file, which is used to configure the environment in which the building occurs. For _team 1_, the build is mainly used for running tests, but this not the case for teams 2 and 3. For _team 2_, the build/pipeline is also their project, and in addition to testing, it also runs several data processing tasks. For _team 3_, in addition to some tests, the build/pipeline is responsible for generating the web-based report that customers can interact with. For _team 4_, all types of tests are included.

**Make the build self-testing:** Developers on all four teams include tests in the automated build. Unit tests are the most common way of testing changes made to the projects developers work on. However, one participant indicated that compilers used to build the projects are also considered a form of testing, in the sense that compilation results indicate whether or not a change will work. For _team 1_, there are additional regression and integration tests that happen on the mainline (but not at the pull request level) to increase build speed. Once a change passes all these tests, it is deployed to the scratch environment where team members can conduct manual tests. For _team 4_, the tests can be disabled manually to run a build against the infrastructure and evaluate it.

**Everyone commits to the mainline every day:** For _teams 1, 2, and 4_, commits happen frequently as their projects are either being developed or actively maintained. For _team 3_, considering its monthly release schedule, commits are less frequent, with one participant reporting that commits happen usually once a week or as necessary. Project priority and release schedule determine the development cadence. Across all four teams, a reviewer is required to approve changes before being merged, which can take some time as well.

**Every commit should build the mainline on an integration machine:** For _teams 1, 2, and 4_, builds happen on three levels; locally, on pull requests, and on merges to the mainline. A developer is expected to run tests locally, and once they are comfortable with the changes they have made, they submit a pull request. The pull request triggers a build on Jenkins, which runs these tests again. Finally, when the pull request is approved and merged into the mainline, another build is triggered which runs the previous tests, in addition to any regression and integration tests that may have been held back to optimize build speed or to test the infrastructure as was mentioned earlier. For _team 3_, due to the monthly nature of the releases, as well as the long duration of builds, builds are triggered manually.

**Keep the build fast:** For _team 1_, builds usually take less than 5 minutes. These short builds are accomplished by running both integration and regression tests on the mainline and not on the pull request builds. The short duration provides rapid feedback to developers and reduces context switching between projects. For _teams 2 and 3_, builds take significantly longer (team 2 around 20 minutes but could go up to 40 minutes, team 3 around 2-3 hours). The long durations are due to the amount of data processing that occurs within the build. For _team 2_, the build/pipeline serves as a vehicle to make data available to their customers, so it also runs data processing tasks in addition to tests. Similarly for _team 3_, processing a month's worth of data also takes a long time. For _team 4_, builds reportedly take around a minute to keep context-switching to an absolute minimum.

**Test in a clone of the production environment:** All four teams in ORG A report using Docker to manage their environments. Using Docker has facilitated keeping development and production environments similar, except for a few small differences which are typically managed by using environment variables to distinguish different configurations. The similarity between the environments guarantees a realistic evaluation is produced by the build.

**Make it easy for anyone to get the latest executable:** The application of this practice was never intentional, rather, it is the result of using Docker to manage application environments. This practice is "unintentionally" applied in _team 1_, which actively uses Docker containers in concert with Kubernetes to deploy their application. It has reduced the complexity of the build process to checking out the mainline and running `docker compose`, as one participant described. For _team 4_, the artifact resulting from the build is added to an S3 bucket and is only accessible by team leads and managers for security reasons.

**Ensure that system state and changes are visible:** All teams report using Slack to communicate build notifications. Whenever a build is triggered, a notification is posted in a dedicated Slack channel with that build's result and an email is sent whenever a pull request is created. It is then up to team members to follow up on the notification. The team members then check BitBucket to see which commit corresponds to the failing build, then investigate Jenkins to see why the build failed (not necessarily in that order, but these are the sources of information available to them). Communicating pull request and build notifications facilitates keeping everyone in the loop as to what is being deployed, when, and its status.

**Automate deployment:** All four teams use Jenkins to deploy to production. The process is completely automated, but requires a manual trigger. For _teams 1 and 2_, deployment privileges are the purview of the team lead for the purposes of conducting a pre-deployment sanity check. For _team 3_, deployment is open to anyone who has access to the Jenkins server. For _team 4_, deployment is restricted to the project's team members on the BitBucket repository, again for security reasons.