# Case Profile: ORG B

## Organization Characteristics

- **Domain:** Provides online content via their publishing platform, and advertisement management.
- **Organization Size:** 51-200 employees / Medium according to [StatCanada](https://web.archive.org/web/20200604201858/https://www150.statcan.gc.ca/n1/pub/11f0027m/2011069/part-partie1-eng.htm)
- **Team(s):** 3 (1 whose members we've interviewed)
- **Number of Interviewees:** 5
- **Average Team Size:** 8-10
- **Distribution:** Employees are mostly co-located, with a few team members working from abroad.
- **Continuous Status:** ORG B classifies itself as an organization that follows continuous practices. They have incorporated continuous tools and practices into their developer workflow.
- **Continuous Integration Infrastructure:** ORG B uses AWS CodePipeline along as AWS CodeBuild as its primary platform for build automation, with Docker to manage container configuration, and deployments are handled by AWS CloudFormation and production is monitored by AWS CloudWatch.


## Background

In this section, we attempt to provide context around the current workflow, including why ORG B decided to move to the continuous methodology, what we understand to be ORG B's interpretation of the continuous methodology through our interviews, and what makes ORG B unique with respect to other organizations within our study.

### Reasons for adopting the continuous methodology

When the decision was made to adopt the continuous methodology, there were two main reasons: rapid feedback on changes made to the products the team delivers, and efficiency brought about by automation.
From a development perspective, rapid feedback was about cultivating the feeling that the changes they were making had a quick, observable impact on the overall platform they deploy.
From management's perspective, the rapid feedback was more about following agile practices and eliciting feedback from users as often as possible.
Automation would cut down on repetitive manual processes, freeing up developers to focus on issues that required creativity to solve, which typically were of higher priority.

### Interpretation of the continuous methodology

Three out of the five interviewees from ORG B associated the continuous methodology with the use of automated builds and other tools (VCS, pipelines, etc.) that support developer workflow.
One of the three participants elaborated on tests being part of the build, but the response was still tool-centric as opposed to practice-centric.

### Unique aspect: Highly structured workflow

ORG B has a standardized workflow that applies to every component of their platform.
Their pipelines are variations on a master pipeline to ensure repeatability and facilitate component integration into their platform.

### Teams we've interviewed

- **Team 1:** This team focuses on creating and maintaining current web _properties_, which serve as components within their web publishing platform. Each of the properties is a web application in its own right and is treated as a separate project, but they all follow the same workflow as described in the previous point about ORG B's highly structured workflow.

## Current Status

Throughout this section, we go over the reported benefits, possible problems and trade-offs, and recommendations regarding ORG B's current workflow. Afterwards, we go into more detail regarding reported developer workflow as per the interviews we conducted. Finally, we compare that workflow based on the continuous practices we are investigating as originally drafted by [Fowler](https://web.archive.org/web/20200522100521/https://martinfowler.com/articles/originalContinuousIntegration.html).

### Benefits attributed to the continuous practices in place

- Having automated builds cuts down on merging bad code.
- Automated builds ensure consistency and repeatability across different development environments.
- Automated builds encapsulate tacit knowledge about the build process.
- Merging to the mainline daily reduces merge conflicts and helps with rapid feedback from users.
- Triggering builds on pull requests and mainline merges enhances feedback speed.
- Short builds reduce blocking and context switching.
- Docker facilitates access to the end product (executable), which in turn facilitates onboarding.
- Integrating build notifications into Slack facilitates transparency.
- Having developers responsible for deploying their changes to production prevents the loss of context.

### Possible problems related to the practices in place

- Duplicated work effort because of a multi-repo structure and a highly structured workflow.
- Dependency issues due to dependencies being built differently across several projects.
- The pipeline can become a bottleneck if builds fail.
- End-to-end testing and smoke tests are flaky.
- Integration tests are harder to define.
- Effort is required to ensure there is no drift between dev and prod environment configurations.

### Trade-offs encountered when applying a practice

- Trade-off between mono-repo vs. multi-repo structure was not obvious when making the decision.
- End-to-end testing requires internet access, which compromises security.
- Regression testing impacts build time.
- No testing in the deployment pipeline improves deployment time.
- The current value of integration tests does not justify the effort required to include them in the build.
- The available infrastructure limits the amount and types of tests included in the build.

### Recommendations

- A configuration item database can help keep track of configurations, facilitating maintenance and drift minimization.
- Using popular dependency management systems (npm or Pip) when possible should minimize dependency issues. As for internal dependencies, enforcing some form of stable releases can cut down on the issues that come from constantly having to build them from scratch.
- Pipeline bottlenecks due to failure can be solved by making sure all tests run in all builds. If developers have complete feedback on how their changes will interact with the mainline, failures on the mainline can be minimized. Then when failures do occur, they will be indicative of more severe issues. This is, however, a trade-off against build time.
- Flaky tests are a staple of software development. While it is not possible to make concrete suggestions without more data, smoke tests -at the very least- should not be flaky. I would almost argue that smoke tests should give consistent results since they test the most basic type of functionality. The fact that they are flaky indicates that they might be too complex.
- Integration tests _are_ harder to define and implement. They are, nevertheless, a guarantee that all components work well with each other. While there are various strategies (top-down, bottom-up, sandwich) to how these are defined/conducted, one way that generally works (especially if comprehensive testing is not an option) is following application execution paths, mapping out what modules are involved with that execution path, and then planning integration tests accordingly.

### Detailed workflow

We grouped the information below from the team members we interviewed, which allowed us to construct a model of how the organization as a whole applies the continuous practices we are investigating. We find that ORG B mostly applies continuous practices with minimal deviation from the way they were originally drafted. We were told that the other two teams within the organization that we did not have a chance to interview operate in a similar manner.

#### Typical developer workflow

Developers are expected to work on multiple projects at the same time.
Project repositories contain the source code for a particular project, in addition to build configuration files (typically Docker files).
Dependencies, including those created in-house, are stored in their own repositories, and their binaries are pulled from a dependency management system (npm or Nexus) as needed.
Because developers work on multiple projects at the same time, merge conflicts tend to occur more frequently.
However, these are mitigated by the use of branches, and in some cases frequent, small merges.
Once a developer commits their changes, they open a pull request, which triggers a build that runs tests on that pull request.
Whenever new changes are pushed to that pull request, another build is triggered.
Developers are responsible for merging their changes into the mainline and then deploying them.
Once a pull request is merged into the mainline, another build is triggered.
Changes are first deployed to a user-acceptance testing (UAT) environment that is as similar to production as possible.
At scheduled intervals, the latest state of the UAT environment is then deployed to production.
This happens at least once a day.

#### Comparison against Fowler's continuous practices

**Maintaining a single source repository:** All components within the platform are treated as separate projects in a microservice-like architecture. This modularity facilitates maintaining the individual components as well as adding newer components later on should it be necessary. The reason behind the microservice repository organization is that -at the time- the benefits of using a single source repository were not clear. Each project's repository contains that project's source code, in addition to build scripts and other dependency-related files. Where possible, external dependencies are not stored within the project's repository, instead it falls to dependency management tools (such as npm) to fetch and build these dependencies. This has created an issue where the dependency build process can be different for each project and requires extra effort to keep track of. Furthermore, with the highly structured workflow followed, there is some duplicate effort when it comes to maintaining the separate components.

**Automating the build:** As was mentioned previously, the components' pipelines are created from a master pipeline template. All platform components have their own automated builds via AWS CodePipeline. One participant reports on the benefit of having the build status as feedback show up when making a pull request, which cuts down on merging bad code. Automated builds were introduced to achieve consistency and repeatability. They have also -unexpectedly- been found to encapsulate tacit knowledge. A newcomer to a component can simply inspect the build scripts to understand how to build the component, what dependencies are involved, etc. One participant indicated that the automated build that deploys changes to the user-acceptance testing environment (UAT) can become a bottleneck. Because a failing build typically means that the pipeline needs fixing, whoever's responsible sends out a message locking the pipeline until further notice or until the breaking changes are reverted.

**Make the build self-testing:** Unit tests are included for all components. Only unit tests are run on pull request builds. Sometimes regression tests are run on the mainline build, but other types of tests are subject to trade-offs that result in either not including them in a build or not implementing them. ORG B used to include smoke tests, but those were found to be too flaky to be reliable. Regression and integration (end-to-end) tests are typically excluded from the build, either to shorten build time or because the investment required to implement them does not justify the value they would provide. Testing is also limited by the existing infrastructure. ORG B _fully_ applies this practice in so much that tests are included in the build.

**Everyone commits to the mainline every day:** Commits typically happen frequently, with a target turnaround of 1 day per feature. The initial reason to implement this practice was getting rapid feedback, however, one participant reports they experience less merge conflicts because of it.

**Every commit should build the mainline on an integration machine:** Builds occur on two levels; first on the pull request level where developers get a chance to test their changes and make fixes as necessary, then on the mainline level upon merging. Builds on the mainline typically include more tests (regression tests) as was discussed previously. One participant reported that this practice enhances feedback speed. The enhanced feedback speed is also the reason behind its application.

**Keep the build fast:** Builds are reported to take between 4 and 6 minutes. The short duration is meant to reduce blocking developers until a build is finished. This also cuts down on context switching between different tasks until a build is done. Build speed is achieved by making the containers (in which the builds occur) as thin as possible, and by only running lengthy tests when required, as was discussed previously. 

**Test in a clone of the production environment:** Development and production environment configurations boil down to dockerfiles. An environment variable decides which of the two configurations to use during a build. There are generally subtle differences between the two configurations, like the number of running instances. This is typically done to measure the impact changes would have on an actual user. Having to maintain both configurations has increased the maintenance effort required to ensure there is as little drift as possible between the two.

**Make it easy for anyone to get the latest executable:** Docker facilitates the application of this practice. It is reported to have been reduced to a single command `docker compose` in order to get a version of a component up and running. While other components may require additional steps, these have typically been scripted, and thus also reduced. The explicit reason behind investing so much in making the build process itself simple was to facilitate onboarding of new employees and co-ops (ORG B has a high co-op turnover).

**Ensure that system state and changes are visible:** All team members report using Slack to keep track of what component builds are being launched on which pipelines. A notification is posted to a dedicated Slack channel containing the build status. It is then up to team members to follow up on that. This was implemented to facilitate keeping everyone in the loop, which team members report is the case.

**Automate Deployment:** Deployment is automated, albeit triggered manually. Deployments happen on AWS CloudFormation and do not contain tests. While this privilege may not be afforded to co-ops, developers are expected to deploy their own changes. In this sense, a developer is responsible for their changes from the moment they take on the task until the changes are deployed to production. This has created a culture of ownership that facilitates keeping track of change context, ensuring that the most knowledgeable person is always the person fixing the build should it fail. Deployments are manual and occur on a need-to basis depending on the component being changed.