# Case Profile: ORG C

## Organization Characteristics

- **Domain:** Provides an online booking platform that integrates with multiple existing services and caters to many customers worldwide.
- **Organization Size:** 51 - 200 employees / Medium according to [StatCanada](https://web.archive.org/web/20200604201858/https://www150.statcan.gc.ca/n1/pub/11f0027m/2011069/part-partie1-eng.htm)
- **Team(s):** 3 (whose members we've interviewed)
- **Number of Interviewees:** 8
- **Average Team Size:** 6-12
- **Distribution:** Employees are co-located.
- **Continuous Status:** ORG C classifies itself as moving towards being an organization that follows continuous practices. They have incorporated continuous tools and practices into their developer workflow.


## Background

In this section, we attempt to provide context around the current workflow, including why ORG C decided to move to the continuous methodology, what we understand to be ORG C's interpretation of the continuous methodology through our interviews, and what makes ORG C unique with respect to other organizations within our study.

### Reasons for adopting the continuous methodology

Due to organization growth, it was felt that automation would alleviate some of the pressure that comes with a larger, more complex product. Automation would also make the development workflow more efficient and have a positive impact on product quality.

### Interpretation of the continuous methodology

Upon asking the interviewees whether they were familiar with continuous practices, 3 indicated they were not, 4 gave tool-centric responses revolving around the level of automation they interact with during the day, and 1 associated continuous practices with commit patterns.

### Unique aspect: Single product organization

ORG C's business model is built around a single product, for which there is a dedicated development team. Other teams within the organization handle different aspects of the same product, but the predominant workflow is that of the development team. This is what we focus on throughout this report.

### Teams we've interviewed

- **Team 1:** The development team focuses primarily on adding new features to the organization's main product in response to user stories generated by product managers. They are also responsible for maintaining the product as well as dealing with any issues that may arise during operation.
- **Team 2:** The configuration team focuses on creating different product configurations to suit customer needs. They are not involved with development activities, rather they focus more on customizing the user-facing portion of the product.
- **Team 3:** The support team focuses on handling user-reported issues and providing technical support. They are also not involved with development activities.

## Current Status

Throughout this section, we go over the reported benefits, possible problems and trade-offs, and recommendations regarding ORG C's current workflow. Afterwards, we go into more detail regarding the reported developer workflow as per the interviews we conducted. Finally, we compare that workflow based on the continuous practices we are investigating as originally drafted by [Fowler](https://web.archive.org/web/20200522100521/https://martinfowler.com/articles/originalContinuousIntegration.html).

### Benefits attributed to the continuous practices in place

- Higher product consistency across different environments due to the unified build environment.
- Improves the development experience by delegating repeatable tasks to automation.
- Automation increases velocity.
- Containerizing everything facilitates onboarding.

### Possible problems related to the practices in place

- Too many testing gates (with the majority of them being manual due to the product's user-facing nature) may heavily slow down the process.
- Resource constraints limit how many builds can run concurrently, creating a bottleneck or sometimes ignoring older builds in favor of more current ones.
- The more customizable a product is, the harder it is to simulate production conditions.

### Trade-offs encountered when applying a practice

- Build duration vs. test execution paradigm.
- Test execution paradigm vs. infrastructure resources.
- Build concurrency vs. infrastructure resources.

### Recommendations

- There are three levels of manual tests occurring before a change is deployed to production (not including the pre-deployment test). Cutting down on these gates or even automating some of the manual testing may increase velocity.
- Resource constraints are typically an issue that has to be considered from multiple angles (including business). However, it seems -at the very least- that tests need not be run serially. Some test suites allow tests to run in parallel, reducing build time.
- Builds should not be dropped in favor of more recent ones as this creates an additional set of variables to test, almost reverting the process back to a big bang integration.
- To get around resource restrictions, a possible solution is to use BitBucket's own CI system for running builds on pull requests, while utilizing the in-house infrastructure for mainline and deployment builds.
- Should the infrastructure permit, running one staging instance per customer could be a way to ensure that test environments are always similar to production. While this introduces some maintenance overhead, it can dramatically reduce build time (if the application is being tested on different configurations within the build). It also ensures customer-specific consistency when rolling out new features. This concept is essentially extremely similar to _digital twins_.

### Detailed workflow

We grouped the information below from all three teams whose members we've interviewed, allowing us to construct a model of how the organization as a whole applies the continuous practices we are investigating. We find that ORG C applies continuous practices with some deviation from the way they were originally drafted.

#### Typical developer workflow

Developers primarily work on a single project, with exceptions based on whether internal tooling is required.
The main project repository contains its source code, in addition to build configuration files.
Dependencies (external and in-house) are kept separately, with external dependencies being the responsibility of dependency tools appropriate to the project.
Similarly to ORG B, merge conflicts occur frequently.
Once a developer commits their changes, they open a pull request.
This triggers a build that runs tests on that pull request, and any further changes to that pull request trigger additional builds (new build per push).
Once a pull request has been merged into the mainline, another build is triggered.
Completed and merged tasks are marked for review by dedicated quality assurance personnel, where they undergo another round of testing.
Then they are tested by product managers.
Once a day, a deployment occurs where the state of the mainline is deployed to production.

#### Comparison against Fowler's continuous practices

**Maintaining a single source repository:** The organization's main product (as well as other relevant dependencies) exist in one central repository. One participant mentions there are a few internally developed tools and dependencies that live in separate repositories. It is not clear why ORG C decided to follow the mono repo paradigm, but participants reported that this was the operating model when the organization was created. Another participant mentioned that elements of the product are so interconnected that -at the current time- it does not make sense to separate them into their own repositories.

**Automating the build:** The development team runs automated builds on Jenkins. These were introduced to improve the development experience by delegating repeatable tasks to automation and increase velocity. Participants reported that they also guarantee consistency when running builds across different environments. The automated builds also guarantee that tests are executed once changes are pushed (either to a branch or to the mainline) and are not forgotten.

**Make the build self-testing:** Testing happens on several levels to ensure high quality. First, developers are expected to test their changes locally. Then, when they initiate a pull request, a build runs that tests their changes (unit, regression, and integration tests). Another build runs once these changes are merged into the mainline. A developer then marks a task as ready for quality assurance (QA), signaling QA to test the feature as per its definition in the user story. Afterwards, the product manager is expected to test that feature again.

**Everyone commits to the mainline every day:** Daily commits to the mainline are encouraged because this minimizes merge conflicts. However, because developers typically work on feature branches and implement tasks of varying sizes, daily commits are not always feasible.

**Every commit should build the mainline on an integration machine:** As was mentioned previously, typically every merge to the mainline triggers a build with the exception of concurrent builds. If two changes are merged within a short interval of time, the older build is scrapped in favor of the newer one, to conserve resources.

**Keep the build fast:** Builds take on average between 10 and 20 minutes. The longer duration is due to the large number of tests that are run within the build, which one participant noted run serially. Coupled with the large number of developers frequently merging changes and limited infrastructure resources to run builds, this has the potential to become a bottleneck. While there is a build queuing system in place, this is still a problem.

**Test in a clone of the production environment:** While the environment similarity is encouraged to avoid introducing environment-related bugs, there are still differences on two levels. The first level is in environment configuration, where the configuration of the environment itself changes based on whether the product is being run in a development or production environment. The second level is that of product configuration. As was mentioned previously, ORG C's product is customizable to suit customer needs. Each customer has a specific (almost unique) configuration that affects what features are available, how some of them function, and so on.

**Make it easy for anyone to get the latest executable:** Over the past few years, ORG C has adopted Docker to containerize development environments. The reason behind this was to ensure a common build environment across all developer systems. The other reason was to facilitate onboarding. A developer need simply run the container and within a few minutes a complete development environment is made available for them to begin coding.

**Ensure that system state and changes are visible:** Developers rely mainly on Jenkins or BitBucket to inform them of build status. Build status is also reported in a dedicated Slack channel. While this is mainly for members of the development team to keep track of what the state of the mainline build is, other teams also have access to the information. However, it is not clear how much they keep track of build status and development workflow due to their varying roles throughout the organization.

**Automate deployment:** Deployment is automated and happens via Jenkins, but the deployment process is triggered manually once a day (if not twice a day). Developers attempt to avoid deployments on Fridays so as to minimize the risk of introducing bugs in production during the weekend. At the time of the interviews, ORG C was in the process of migrating the production environment(s) to Kubernetes. Not all developers can deploy due to security reasons. ORG C team members describe a previously convoluted _secure_ deployment process which has now been improved. There is also some preparation required before deployment, typically testing and writing up release notes.